//
// file_name = HSListBox.pas
//
// Notes:
//  - created Oct 16, 1997 Gregor Smith
//  - a list box that adds a Horizontal scroll bar as needed
//  - there are no new properties or methods, it should all happen
//  automatically
//  - this is freeware, please feel free to use it although I there is
//  no warrantee either implied or expressed. Enough of the lawyer crap,
//  if you have any questions please feel free to drop me
//  email at gregor_smith@bigfoot.com and I will try to help you out.
//  Let me know if this has been of any use to you.
//
// This is a 32 bit component so it will only work on Delphi 2, 3 and
// CBuilder. Install it as you would install any other component.
// If you don't like the tab it appears on, change the register procedure


unit HSListBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  THSListBox = class(TListBox)
  private
    { Private declarations }
    MaxExtent: integer;
    function GetStringWidth( s: string ): integer;
    procedure LBAddString( var Message: TMessage ); message LB_ADDSTRING;
    procedure LBInsertString( var Message: TMessage ); message LB_INSERTSTRING;
    procedure LBDeleteString( var Message: TMessage ); message LB_DELETESTRING;
    procedure LBResetContent( var Message: TMessage ); message LB_RESETCONTENT;
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
  end;

procedure Register;

implementation

function THSListBox.GetStringWidth( s: string ): integer;
var
  hDCTemp: HDC;
  hFontOld, hFontTemp: HFONT;
  tm: TTextMetric;
  size: TSize;
begin
  // get the temporary dc for the list box
  hDCTemp := GetDC( Handle );
  // Get a handle to the font used in the list box
  hFontTemp := SendMessage( Handle, WM_GETFONT, 0, 0 );
  // Select the list box font into the temporary DC
  hFontOld := SelectObject( hDCTemp, hFontTemp );

  GetTextMetrics( hDCTemp, tm );
  GetTextExtentPoint32( hDCTemp, PChar(s), Length(s), size );

  // Restore Original Font
  SelectObject( hDCTemp, hFontOld );
  ReleaseDC( Handle, hDCTemp );
  // calculate the width
  Result := size.cx + tm.tmAveCharWidth;
end;

procedure THSListBox.LBAddString( var Message: TMessage );
var
  StrExtent: integer;
begin
  StrExtent := GetStringWidth( PChar(Message.lParam) );
  if StrExtent > MaxExtent then
  begin
    MaxExtent := StrExtent;
    SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
  end;
  inherited;
end;

procedure THSListBox.LBInsertString( var Message: TMessage );
var
  StrExtent: integer;
begin
  StrExtent := GetStringWidth( PChar(Message.lParam) );
  if StrExtent > MaxExtent then
  begin
    MaxExtent := StrExtent;
    SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
  end;
  inherited;
end;

procedure THSListBox.LBDeleteString( var Message: TMessage );
var
  StrExtent: integer;
  Index: Integer;
  i : Integer;
begin
  Index := Message.wParam;
  StrExtent := GetStringWidth( PChar(Items[ Index ]) );
  if StrExtent = MaxExtent then
  begin
    MaxExtent := 0;
    for i:= 0 to Items.Count - 1 do begin
      if i <> Index then begin
          StrExtent := GetStringWidth( PChar(Items[ i ]) );
          if StrExtent > MaxExtent then begin
            MaxExtent := StrExtent;
          end;
      end;
    end;
    SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
  end;
  inherited;
end;

procedure THSListBox.LBResetContent( var Message: TMessage );
begin
    MaxExtent := 0;
    SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
    inherited;
end;

procedure Register;
begin
  RegisterComponents('Sirius', [THSListBox]);
end;

end.
